---
-- LiveCropScanner
-- Specialization to get live precision farming data usind a field scanner
--
-- @author John Deere 6930
-- @date 28/01/2021
-- @version 1.0
--
LiveCropScanner = {};

LiveCropScanner.MOD_DIRECTORY = g_currentModDirectory;
LiveCropScanner.MOD_NAME = g_currentModName;

function LiveCropScanner.initSpecialization()
    g_workAreaTypeManager:addWorkAreaType("liveCropScanner", false);
end;

function LiveCropScanner.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(WorkArea, specializations) and SpecializationUtil.hasSpecialization(TurnOnVehicle, specializations) and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

function LiveCropScanner.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "getIsPrecisionFarmingActive", LiveCropScanner.getIsPrecisionFarmingActive);
    SpecializationUtil.registerFunction(vehicleType, "processLiveCropScannerArea", LiveCropScanner.processLiveCropScannerArea);
end;

function LiveCropScanner.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "loadWorkAreaFromXML", LiveCropScanner.loadWorkAreaFromXML);
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "doCheckSpeedLimit", LiveCropScanner.doCheckSpeedLimit)
end;

function LiveCropScanner.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", LiveCropScanner);
    SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", LiveCropScanner);
    SpecializationUtil.registerEventListener(vehicleType, "onUpdateTick", LiveCropScanner);
    SpecializationUtil.registerEventListener(vehicleType, "onStopAnimation", LiveCropScanner);
    SpecializationUtil.registerEventListener(vehicleType, "onTurnedOn", LiveCropScanner);
    SpecializationUtil.registerEventListener(vehicleType, "onTurnedOff", LiveCropScanner);
end;

function LiveCropScanner:getIsPrecisionFarmingActive()
    local retVal = nil;
    if _G["FS19_precisionFarming"] ~= nil and _G["FS19_precisionFarming"].g_precisionFarming ~= nil then
        retVal = _G["FS19_precisionFarming"].g_precisionFarming;
    end;
    return retVal;
end;

function LiveCropScanner:onLoad(savegame)
    self.spec_liveCropScanner = self[("spec_%s.liveCropScanner"):format(LiveCropScanner.MOD_NAME)];
    local spec = self.spec_liveCropScanner;

    spec.needsUpdate = false;
    spec.needsDelayedUpdate = false;
    
    spec.scanUpdateTimer = 0;
    spec.scanUpdateTimerIntervalMin = 250; -- ms
    spec.scanUpdateTimerIntervalMax = 1500; -- ms
    spec.scanUpdateTimerInterval = spec.scanUpdateTimerIntervalMax - spec.scanUpdateTimerIntervalMin;
    
    spec.turnedOnAnimations = {};
    local i = 0;
    while true do
        local baseKey = string.format("vehicle.turnOnVehicle.turnedOnAnimation(%d)", i);
        if not hasXMLProperty(self.xmlFile, baseKey) then
            break;
        end;
        local entry = {};
        local name = getXMLString(self.xmlFile, baseKey.."#name");
        if name ~= nil then
            entry.name = name;
            entry.timeOnStop = Utils.getNoNil(getXMLFloat(self.xmlFile, baseKey.."#timeOnStop"), 0.0);
            table.insert(spec.turnedOnAnimations, entry);
        end;
        i = i + 1;
    end;
end;

function LiveCropScanner:onPostLoad()
    local spec = self.spec_liveCropScanner;
    spec.precisionFarming = self:getIsPrecisionFarmingActive();
    if spec.precisionFarming ~= nil then
        spec.soilMap = spec.precisionFarming.soilMap;
        spec.nitrogenMap = spec.precisionFarming.nitrogenMap;
    end;
end;

function LiveCropScanner:onTurnedOn()
    local spec = self.spec_liveCropScanner;
    if spec.precisionFarming ~= nil then
        if spec.nitrogenMap ~= nil then
            spec.nitrogenMap:setRequireMinimapDisplay(true, self);
        end;
    end;
end;
function LiveCropScanner:onTurnedOff()
    local spec = self.spec_liveCropScanner;
    if spec.precisionFarming ~= nil then
        if spec.nitrogenMap ~= nil then
            spec.nitrogenMap:setRequireMinimapDisplay(false, self);
        end;
    end;
end;

function LiveCropScanner:onUpdateTick(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
    local spec = self.spec_liveCropScanner;
    if spec.precisionFarming ~= nil then
        if spec.needsUpdate or spec.needsDelayedUpdate then
            if spec.needsDelayedUpdate then
                -- update minimap 1 frame later
                if spec.nitrogenMap ~= nil then
                    spec.nitrogenMap:setMinimapRequiresUpdate(true);
                end;
                spec.needsDelayedUpdate = false;
            end;
            if spec.needsUpdate then
                if spec.soilMap ~= nil then
                    spec.soilMap:uncoverAnalysedArea(farmId);
                end;
                spec.needsDelayedUpdate = true;
                spec.needsUpdate = false;
            end;
        end;
    end;
end;

function LiveCropScanner:processLiveCropScannerArea(workArea, dt)
    local spec = self.spec_liveCropScanner;
    if spec.precisionFarming ~= nil then
        if spec.soilMap ~= nil then
            if spec.scanUpdateTimer < g_currentMission.time then
                local startWorldX, _, startWorldZ = getWorldTranslation(workArea.start);
                local widthWorldX, _, widthWorldZ = getWorldTranslation(workArea.width);
                local heightWorldX, _, heightWorldZ = getWorldTranslation(workArea.height);
                
                local state = nil;
                local farmId = g_currentMission:getFarmId()
                spec.soilMap:analyseArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, state, farmId);
                
                spec.needsUpdate = true;
                
                -- updateTimer in relation to lastSpeed
                local lastSpeed = self:getLastSpeed(false);                
                local interval = math.max(spec.scanUpdateTimerIntervalMin, math.min(spec.scanUpdateTimerIntervalMax - (spec.scanUpdateTimerInterval * lastSpeed/45), spec.scanUpdateTimerIntervalMax));                
                spec.scanUpdateTimer = g_currentMission.time + interval;
            end;
        end;
    end;
    
    return 0, 0;
end;

function LiveCropScanner:loadWorkAreaFromXML(superFunc, workArea, xmlFile, key)
    local value = superFunc(self, workArea, xmlFile, key);
    if workArea.type == WorkAreaType.DEFAULT then
        workArea.type = WorkAreaType.LIVECROPSCANNER;
    end;
    return value;
end;

function LiveCropScanner:onStopAnimation(name)
    local spec = self.spec_liveCropScanner
    for _, animation in pairs(spec.turnedOnAnimations) do
        if animation.name == name then
            if self:getAnimationTime(name) ~= animation.timeOnStop then
                self:setAnimationTime(name, animation.timeOnStop, true);
            end;
        end;
    end;    
end;

function LiveCropScanner:doCheckSpeedLimit(superFunc)
    return superFunc(self) or self:getIsTurnedOn();
end;